@@ -47,11 +47,17 @@ icd9PoaChoices <- c("yes", "no", "notYes", "notNo")
 #' @param isShortMapping Same as isShort, but applied to \code{icd9Mapping}
 #'   instead of \code{icd9df}. All the codes in a mapping should be of the same
 #'   type, i.e. short or decimal.
+#' @param ... further arguments e.g. chunkSize and ompChunkSize pass to the C++
+#'   function
 #' @details There is a change in behavior from previous versions. The visitId
 #'   column is (implicitly) sorted by using std::set container. Previously, the
 #'   visitId output order was whatever R's \code{aggregate} produced.
+#'
+#'   The threading of the C++ can be controlled using e.g.
+#'   \code{option(icd9.threads = 4)}. If it is not set, the number of cores in
+#'   the machine is used.
 #' @examples
-#'   pts = data.frame(visitId = c("2", "1", "2", "3", "3"),
+#'   pts <- data.frame(visitId = c("2", "1", "2", "3", "3"),
 #'                    icd9 = c("39891", "40110", "09322", "41514", "39891"))
 #'    icd9ComorbidShort(pts, ahrqComorbid) # visitId is now sorted
 #' @export
@@ -60,28 +66,76 @@ icd9Comorbid <- function(icd9df,
                          visitId = "visitId",
                          icd9Field = "icd9",
                          isShort = icd9GuessIsShort(icd9df[[icd9Field]]),
-                         isShortMapping = icd9GuessIsShort(icd9Mapping)) {
-
-  checkmate::checkDataFrame(icd9df, min.cols = 2)
-  checkmate::checkList(icd9Mapping, types = "character", any.missing = FALSE, min.len = 1)
-  #checkmate::checkString(visitId)
-  visitId <- as.character(visitId)
-  checkmate::checkString(icd9Field)
-  checkmate::checkLogical(isShort, any.missing = FALSE, len = 1)
-  checkmate::checkLogical(isShortMapping, any.missing = FALSE, len = 1)
+                         isShortMapping = icd9GuessIsShort(icd9Mapping),
+                         return.df = FALSE, ...) {
+  # TODO: allow factors for icd9df fields
+  checkmate::assertDataFrame(icd9df, min.cols = 2)
+  checkmate::assertList(icd9Mapping, any.missing = FALSE, min.len = 1,
+                        types = c("character", "factor"), names = "unique")
+  checkmate::assertString(visitId)
+  checkmate::assertString(icd9Field)
+  checkmate::assertFlag(isShort)
+  checkmate::assertFlag(isShortMapping)
   stopifnot(visitId %in% names(icd9df), icd9Field %in% names(icd9df))
 
   if (!isShort)
     icd9df[[icd9Field]] <- icd9DecimalToShort(icd9df[[icd9Field]])
 
+  icd9Mapping <- lapply(icd9Mapping, asCharacterNoWarn)
+
   if (!isShortMapping)
     icd9Mapping <- lapply(icd9Mapping, icd9DecimalToShort)
 
-  # return via call to the C++ function:
-  icd9ComorbidShort(icd9df, icd9Mapping, visitId, icd9Field)
+  # new stragegy is to start with a factor for the icd codes in icd9df, recode
+  # (and drop superfluous) icd codes in the mapping, then do very fast match on
+  # integer without need for N, V or E distinction. Char to factor conversion in
+  # R is very fast.
 
+  if (!is.factor(icd9df[[icd9Field]]))
+    icd9df[[icd9Field]] <- as.factor(icd9df[[icd9Field]])
+
+  # we need to convert to string and group these anyway, and much easier and
+  # pretty quick to do it here:
+  icd9VisitWasFactor <- is.factor(icd9df[[visitId]])
+  if (icd9VisitWasFactor) ivLevels <- levels(icd9df[[visitId]])
+  icd9df[[visitId]] <- asCharacterNoWarn(icd9df[[visitId]])
+
+  # again, R is very fast at creating factors from a known set of levels
+  icd9Mapping <- lapply(icd9Mapping, function(x) {
+    f <- factor(x, levels(icd9df[[icd9Field]]))
+    f[!is.na(f)]
+  })
+
+  # now we also benefit from only have factor levels in the mapping which appear
+  # in the diagnoses we are going to be examining. From the C++ perspective, we
+  # can now do pure integer matching for icd9 codes. Only string manip becomes
+  # (optionally) defactoring the visitId for the matrix row names.
+
+  # return via call to the C++ function:
+  #icd9ComorbidShort(icd9df, icd9Mapping, visitId, icd9Field)
+
+  threads <- getOption("icd9.threads", getOmpCores())
+
+  if (return.df) {
+    mat <- icd9ComorbidShortCpp(icd9df, icd9Mapping, visitId, icd9Field, threads = threads)
+    if (icd9VisitWasFactor)
+      rownm <- factor(x = rownames(mat), levels = ivLevels)
+    else
+      rownm <- rownames(mat)
+    df.out <- cbind(rownm, as.data.frame(mat), stringsAsFactors = icd9VisitWasFactor)
+    names(df.out)[1] <- visitId
+    # perhaps leave (duplicated) rownames which came from the matrix:
+    rownames(df.out) <- NULL
+    return(df.out)
+  }
+  icd9ComorbidShortCpp(icd9df, icd9Mapping, visitId, icd9Field, threads = threads)
 }
 
+#' @rdname icd9Comorbid
+#' @export
+icd9ComorbidShort <- function(...)
+  icd9Comorbid(..., isShort = TRUE)
+
 #' @rdname icd9Comorbid
 #' @param abbrevNames  single locical value that defaults to \code{TRUE}, in
 #'   which case the ishorter human-readable names stored in e.g.
@@ -91,34 +145,32 @@ icd9Comorbid <- function(icd9df,
 #'   Elixhauser, you can't have uncomplicated and complicated diabetes both
 #'   flagged.
 #' @export
-icd9ComorbidAhrq <- function(icd9df,
-                             isShort = icd9GuessIsShort(icd9df[[icd9Field]]),
-                             visitId = "visitId",
-                             icd9Field = "icd9",
-                             abbrevNames = TRUE,
+icd9ComorbidAhrq <- function(..., abbrevNames = TRUE,
                              applyHierarchy = TRUE) {
+  checkmate::assertFlag(abbrevNames)
+  checkmate::assertFlag(applyHierarchy)
 
-  cbd <- icd9Comorbid(icd9df = icd9df, icd9Mapping = icd9::ahrqComorbid,
-                      visitId = visitId, icd9Field = icd9Field,
-                      isShort = isShort)
+  cbd <- icd9Comorbid(..., icd9Mapping = icd9::ahrqComorbid)
   if (applyHierarchy) {
 
     # Use >0 rather than logical - apparently faster, and future proof against
     # change to binary from logical values in the matirx.
-    cbd[cbd[["Mets"]] > 0, "Tumor"] <- FALSE
-    cbd[cbd[["DMcx"]] > 0, "DM"] <- FALSE
-    cbd[["HTN"]] <- cbd[["HTN"]] + cbd[["HTNcx"]] > 0
-    cbd[["HTNcx"]] <- NULL
+    cbd[cbd[, "Mets"] > 0, "Tumor"] <- FALSE
+    cbd[cbd[, "DMcx"] > 0, "DM"] <- FALSE
+    cbd[, "HTN"] <- (cbd[, "HTN"] + cbd[, "HTNcx"]) > 0
+
+    # drop HTNcx without converting to vector if matrix only has one row (drop=FALSE)
+    cbd <- cbd[, -which(colnames(cbd) == "HTNcx"), drop = FALSE]
 
     if (abbrevNames)
-      names(cbd)[-1] <- icd9::ahrqComorbidNamesAbbrev
+      colnames(cbd)[cr(cbd)] <- icd9::ahrqComorbidNamesAbbrev
     else
-      names(cbd)[-1] <- icd9::ahrqComorbidNames
+      colnames(cbd)[cr(cbd)] <- icd9::ahrqComorbidNames
   } else {
     if (abbrevNames)
-      names(cbd)[-1] <- icd9::ahrqComorbidNamesHtnAbbrev
+      colnames(cbd)[cr(cbd)] <- icd9::ahrqComorbidNamesHtnAbbrev
     else
-      names(cbd)[-1] <- icd9::ahrqComorbidNamesHtn
+      colnames(cbd)[cr(cbd)] <- icd9::ahrqComorbidNamesHtn
   }
   cbd
 }
@@ -129,55 +181,40 @@ icd9ComorbidAhrq <- function(icd9df,
 #'   this is probaably useful, in general and is essential when calculating the
 #'   Charlson score.
 #' @export
-icd9ComorbidQuanDeyo <- function(icd9df,
-                                 visitId = "visitId",
-                                 icd9Field = "icd9",
-                                 isShort = icd9GuessIsShort(icd9df[[icd9Field]]),
-                                 abbrevNames = TRUE,
+icd9ComorbidQuanDeyo <- function(..., abbrevNames = TRUE,
                                  applyHierarchy = TRUE) {
-  cbd <- icd9Comorbid(icd9df = icd9df, icd9Mapping = icd9::quanDeyoComorbid,
-                      visitId = visitId,
-                      icd9Field = icd9Field,
-                      isShort = isShort
-  )
+  checkmate::assertFlag(abbrevNames)
+  checkmate::assertFlag(applyHierarchy)
+  cbd <- icd9Comorbid(..., icd9Mapping = icd9::quanDeyoComorbid)
   if (applyHierarchy) {
-
     # Use >0 rather than logical - apparently faster, and future proof against
     # change to binary from logical values in the matirx.
-    cbd[cbd[["Mets"]] > 0, "Cancer"] <- FALSE
-    cbd[cbd[["DMcx"]] > 0, "DM"] <- FALSE
-    cbd[cbd[["LiverSevere"]] > 0, "LiverMild"] <- FALSE
+    cbd[cbd[, "Mets"] > 0, "Cancer"] <- FALSE
+    cbd[cbd[, "DMcx"] > 0, "DM"] <- FALSE
+    cbd[cbd[, "LiverSevere"] > 0, "LiverMild"] <- FALSE
   }
   if (abbrevNames)
-    names(cbd)[-1] <- icd9::charlsonComorbidNamesAbbrev
+    colnames(cbd)[cr(cbd)] <- icd9::charlsonComorbidNamesAbbrev
   else
-    names(cbd)[-1] <- icd9::charlsonComorbidNames
+    colnames(cbd)[cr(cbd)] <- icd9::charlsonComorbidNames
 
   cbd
 }
 
 #' @rdname icd9Comorbid
 #' @export
-icd9ComorbidQuanElix <- function(icd9df,
-                                 visitId = "visitId",
-                                 icd9Field = "icd9",
-                                 isShort = icd9GuessIsShort(icd9df[[icd9Field]]),
-                                 abbrevNames = TRUE,
+icd9ComorbidQuanElix <- function(..., abbrevNames = TRUE,
                                  applyHierarchy = TRUE) {
-  cbd <- icd9Comorbid(icd9df = icd9df,
-                      icd9Mapping = icd9::quanElixComorbid,
-                      visitId = visitId,
-                      icd9Field = icd9Field,
-                      isShort = isShort)
+  checkmate::assertFlag(abbrevNames)
+  checkmate::assertFlag(applyHierarchy)
+  cbd <- icd9Comorbid(..., icd9Mapping = icd9::quanElixComorbid)
   if (applyHierarchy) {
-
-    # Use >0 rather than logical - apparently faster, and future proof against
-    # change to binary from logical values in the matirx.
-    cbd[cbd[["Mets"]] > 0, "Tumor"] <- FALSE
-    cbd[cbd[["DMcx"]] > 0, "DM"] <- FALSE
+    cbd[cbd[, "Mets"] > 0, "Tumor"] <- FALSE
+    cbd[cbd[, "DMcx"] > 0, "DM"] <- FALSE
     # combine HTN
-    cbd[["HTN"]] <- cbd[["HTN"]] + cbd[["HTNcx"]] > 0
-    cbd[["HTNcx"]] <- NULL
+    cbd[, "HTN"] <- (cbd[, "HTN"] + cbd[, "HTNcx"]) > 0
+    # drop HTNcx without converting to vector if matrix only has one row (drop=FALSE)
+    cbd <- cbd[, -which(colnames(cbd) == "HTNcx"), drop = FALSE]
 
     # if we didn't apply the hierarchy, we have to use the naming scheme with
     # HTN separated out:
@@ -189,45 +226,41 @@ icd9ComorbidQuanElix <- function(icd9df,
     # the comorbidities:
 
     if (abbrevNames)
-      names(cbd)[-1] <- icd9::quanElixComorbidNamesAbbrev
+      colnames(cbd)[cr(cbd)] <- icd9::quanElixComorbidNamesAbbrev
     else
-      names(cbd)[-1] <- icd9::quanElixComorbidNames
+      colnames(cbd)[cr(cbd)] <- icd9::quanElixComorbidNames
   } else {
     if (abbrevNames)
-      names(cbd)[-1] <- icd9::quanElixComorbidNamesHtnAbbrev
+      colnames(cbd)[cr(cbd)] <- icd9::quanElixComorbidNamesHtnAbbrev
     else
-      names(cbd)[-1] <- icd9::quanElixComorbidNamesHtn
+      colnames(cbd)[cr(cbd)] <- icd9::quanElixComorbidNamesHtn
   }
   cbd
 }
 
 #' @rdname icd9Comorbid
 #' @export
-icd9ComorbidElix <- function(icd9df,
-                             visitId = "visitId",
-                             icd9Field = "icd9",
-                             isShort = icd9GuessIsShort(icd9df[[icd9Field]]),
-                             abbrevNames = TRUE,
-                             applyHierarchy = TRUE) {
-  cbd <- icd9Comorbid(icd9df = icd9df,
-                      icd9Mapping = icd9::elixComorbid,
-                      visitId = visitId,
-                      icd9Field = icd9Field,
-                      isShort = isShort)
+icd9ComorbidElix <- function(..., abbrevNames = TRUE, applyHierarchy = TRUE) {
+  checkmate::assertFlag(abbrevNames)
+  checkmate::assertFlag(applyHierarchy)
+  cbd <- icd9Comorbid(..., icd9Mapping = icd9::elixComorbid)
   if (applyHierarchy) {
-    cbd[cbd[["Mets"]] > 0, "Tumor"] <- FALSE
-    cbd[cbd[["DMcx"]] > 0, "DM"] <- FALSE
-    cbd[["HTN"]] <- cbd[["HTN"]] + cbd[["HTNcx"]] > 0
-    cbd[["HTNcx"]] <- NULL
+    cbd[cbd[, "Mets"] > 0, "Tumor"] <- FALSE
+    cbd[cbd[, "DMcx"] > 0, "DM"] <- FALSE
+    cbd[, "HTN"] <- (cbd[, "HTN"] + cbd[, "HTNcx"]) > 0
+
+    # drop HTNcx without converting to vector if matrix only has one row (drop=FALSE)
+    cbd <- cbd[, -which(colnames(cbd) == "HTNcx"), drop = FALSE]
+
     if (abbrevNames)
-      names(cbd)[-1] <- icd9::elixComorbidNamesAbbrev
+      colnames(cbd)[cr(cbd)] <- icd9::elixComorbidNamesAbbrev
     else
-      names(cbd)[-1] <- icd9::elixComorbidNames
+      colnames(cbd)[cr(cbd)] <- icd9::elixComorbidNames
   } else {
     if (abbrevNames)
-      names(cbd)[-1] <- icd9::elixComorbidNamesHtnAbbrev
+      colnames(cbd)[cr(cbd)] <- icd9::elixComorbidNamesHtnAbbrev
     else
-      names(cbd)[-1] <- icd9::elixComorbidNamesHtn
+      colnames(cbd)[cr(cbd)] <- icd9::elixComorbidNamesHtn
   }
   cbd
 }
@@ -282,11 +315,12 @@ icd9ComorbiditiesQuanElixhauser <- function(...) icd9ComorbidQuanElix(...)
 #' @export
 icd9DiffComorbid <- function(x, y, names = NULL, x.names = NULL, y.names = NULL,
                              show = TRUE, explain = TRUE) {
-  checkmate::checkList(x, min.len = 1, any.missing = FALSE)
-  checkmate::checkList(y, min.len = 1, any.missing = FALSE,
-                       types = c("character", "numeric", "integer"))
-  checkmate::checkLogical(show, any.missing = FALSE, len = 1)
-  checkmate::checkLogical(explain, any.missing = FALSE, len = 1)
+  checkmate::assertList(x, min.len = 1, any.missing = FALSE,
+                        types = c("character"), names = "unique")
+  checkmate::assertList(y, min.len = 1, any.missing = FALSE,
+                        types = c("character"), names = "unique")
+  checkmate::assertFlag(show)
+  checkmate::assertFlag(explain)
   stopifnot(all(x.names %in% names(x)), all(y.names %in% names(y)))
 
   lapply(x, function(z) stopifnot(is.character(z)))
@@ -299,13 +333,13 @@ icd9DiffComorbid <- function(x, y, names = NULL, x.names = NULL, y.names = NULL,
     x.names <- names
     y.names <- names
   }
-  if (is.null(x.names)) x.names = names(x)
-  if (is.null(y.names)) y.names = names(y)
+  if (is.null(x.names)) x.names <- names(x)
+  if (is.null(y.names)) y.names <- names(y)
 
-  common.names = intersect(x.names, y.names)
+  common.names <- intersect(x.names, y.names)
 
-  x.title = deparse(substitute(x))
-  y.title = deparse(substitute(y))
+  x.title <- deparse(substitute(x))
+  y.title <- deparse(substitute(y))
 
   out <- list();
 
@@ -328,34 +362,37 @@ icd9DiffComorbid <- function(x, y, names = NULL, x.names = NULL, y.names = NULL,
                     x.title, length(only.x), y.title))
         lapply(icd9Explain(only.x, doCondense = TRUE, brief = TRUE, warn = FALSE)[1:5],
                function(s) if (!is.na(s)) cat(sprintf("'%s' ", s)))
-
-        #lapply(only.x[1:5], function(s) if (!is.na(s)) cat(sprintf("%s ", s)))
       }
       if (length(only.y) > 0) {
         cat(sprintf("\n%s has %d codes not in %s. First few are: ",
                     y.title, length(only.y), x.title))
         lapply(icd9Explain(only.y, doCondense = TRUE, brief = TRUE, warn = FALSE)[1:5],
                function(s) if (!is.na(s)) cat(sprintf("'%s' ", s)))
-        #lapply(only.y[1:5], function(s) if (!is.na(s)) cat(sprintf("%s ", s)))
       }
       cat("\n")
     }
   }
   if (show) {
-    cmb.only.x <- setdiff(x.names, y.names)
-    cmb.only.y <- setdiff(y.names, x.names)
+    cmb_only_x <- setdiff(x.names, y.names)
+    cmb_only_y <- setdiff(y.names, x.names)
 
-    if (length(cmb.only.x) > 0) {
+    if (length(cmb_only_x) > 0) {
       cat(sprintf("Comorbidities only defined in %s are: ", x.title))
-      lapply(cmb.only.x, function(s) cat(sprintf("%s ", s)))
+      lapply(cmb_only_x, function(s) cat(sprintf("%s ", s)))
       cat("\n")
     }
 
-    if (length(cmb.only.y) > 0) {
+    if (length(cmb_only_y) > 0) {
       cat(sprintf("Comorbidities only defined in %s are: ", y.title))
-      lapply(cmb.only.y, function(s) cat(sprintf("%s ", s)))
+      lapply(cmb_only_y, function(s) cat(sprintf("%s ", s)))
       cat("\n")
     }
   }
   invisible(out)
 }
+
+#' @title get sequence of column indices of comorbidities
+#' @param cbd matrix or data.frame of comorbidities
+#' @keywords internal
+cr <- function(cbd)
+  seq(from = 1 + is.data.frame(cbd), to = ncol(cbd))
